@@ -7,15 +7,31 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
+
 from collections import defaultdict
-from unittest import mock
+from contextlib import ExitStack
 from enum import Enum
 from functools import partial
+from unittest import mock
 
-from apiver_deps import UpPolicy, B2DownloadAction, AbstractSyncEncryptionSettingsProvider, UploadSourceLocalFile, SyncPolicyManager
-from apiver_deps_exception import DestFileNewer, InvalidArgument
-from apiver_deps import KeepOrDeleteMode, NewerFileSyncMode, CompareVersionMode
 import pytest
+from apiver_deps import (
+    AbstractSyncEncryptionSettingsProvider,
+    B2DownloadAction,
+    CompareVersionMode,
+    CopySource,
+    FileVersion,
+    IncrementalHexDigester,
+    KeepOrDeleteMode,
+    NewerFileSyncMode,
+    SyncPolicyManager,
+    UploadSourceLocalFile,
+    UploadSourceLocalFileRange,
+    UpPolicy,
+)
+from apiver_deps_exception import DestFileNewer, InvalidArgument
+
 from ..fixtures.folder import *
 from .fixtures import *
 
@@ -475,7 +491,7 @@ class TestSynchronizer:
             ('b2', 'b2', ['b2_copy(folder/a.txt, id_a_200, folder/a.txt, 200)']),
         ],
     )
-    def test_never(self, synchronizer, src_type, dst_type, expected):
+    def test_newer(self, synchronizer, src_type, dst_type, expected):
         src = self.folder_factory(src_type, ('a.txt', [200]))
         dst = self.folder_factory(dst_type, ('a.txt', [100]))
         self.assert_folder_sync_actions(synchronizer, src, dst, expected)
@@ -722,7 +738,7 @@ class TestSynchronizer:
         with mock.patch.object(B2DownloadAction, '_ensure_directory_existence'):
             try:
                 download_action.do_action(bucket, self.reporter)
-            except:
+            except:  # noqa: E722
                 pass
 
         assert bucket.mock_calls[0] == mock.call.download_file_by_id(
@@ -767,16 +783,17 @@ class TestSynchronizer:
         with mock.patch.object(UploadSourceLocalFile, 'check_path_and_get_size'):
             try:
                 upload_action.do_action(bucket, self.reporter)
-            except:
+            except:  # noqa: E722
                 pass
 
         assert bucket.mock_calls == [
-            mock.call.upload(
+            mock.call.concatenate(
                 mock.ANY,
                 'folder/directory/a.txt',
                 file_info={'src_last_modified_millis': '100'},
                 progress_listener=mock.ANY,
-                encryption=encryption
+                encryption=encryption,
+                large_file_sha1=None,
             )
         ]
 
@@ -870,6 +887,88 @@ class TestSynchronizer:
         expected = ['b2_upload(/dir/a.txt, folder/a.txt, 200)']
         self.assert_folder_sync_actions(synchronizer, src, dst, expected)
 
+    # FIXME: rewrite this test to not use mock.call checks when all of Synchronizers tests are rewritten to test_bucket
+    # style - i.e. with simulated api and fake files returned from methods.
+    @pytest.mark.apiver(from_ver=2)
+    @pytest.mark.parametrize(
+        "local_size,remote_size,local_sha1,local_partial_sha1,remote_sha1,should_be_incremental",
+        [
+            (2000, 1000, "ff" * 20, "aa" * 20, "aa" * 20, True),  # incremental upload possible
+            (2000, 999, "ff" * 20, "aa" * 20, "aa" * 20, False),  # uploaded part too small
+            (2000, 1000, "ff" * 20, "aa" * 20, None, False),  # remote sha unknown
+            (2000, 1000, "ff" * 20, "aa" * 20, "bb" * 20, False),  # remote sha mismatch
+            (2000, 3000, "ff" * 20, "aa" * 20, "bb" * 20, False),  # remote file bigger
+        ]
+    )
+    def test_incremental_upload(
+        self, synchronizer_factory, local_size, remote_size, local_sha1, local_partial_sha1,
+        remote_sha1, should_be_incremental
+    ):
+
+        synchronizer = synchronizer_factory(
+            upload_mode=UploadMode.INCREMENTAL, absolute_minimum_part_size=1000
+        )
+
+        src = self.folder_factory('local', ('a.txt', [200], local_size))
+        dst = self.folder_factory('b2', ('a.txt', [100], remote_size))
+
+        upload_action = next(
+            iter(self._make_folder_sync_actions(synchronizer, src, dst, TODAY, self.reporter))
+        )
+
+        bucket = mock.MagicMock()
+
+        def update_from_stream(self, limit=None):
+            if limit is None:
+                return local_sha1
+            elif limit == remote_size:
+                return local_partial_sha1
+            else:
+                assert False
+
+        def check_path_and_get_size(self):
+            self.content_length = local_size
+
+        with ExitStack() as stack:
+            patches = [
+                mock.patch.object(
+                    UploadSourceLocalFile, 'open', mock.mock_open(read_data='test-data')
+                ),
+                mock.patch.object(IncrementalHexDigester, 'update_from_stream', update_from_stream),
+                mock.patch.object(
+                    UploadSourceLocalFile, 'check_path_and_get_size', check_path_and_get_size
+                ),
+                mock.patch.object(
+                    UploadSourceLocalFile, '_hex_sha1_of_file', return_value=local_sha1
+                ),
+                mock.patch.object(
+                    UploadSourceLocalFileRange, 'check_path_and_get_size', check_path_and_get_size
+                ),
+                mock.patch.object(FileVersion, 'get_content_sha1', return_value=remote_sha1),
+            ]
+            for patch in patches:
+                stack.enter_context(patch)
+
+            upload_action.do_action(bucket, self.reporter)
+
+        assert bucket.mock_calls == [
+            mock.call.concatenate(
+                mock.ANY,
+                'folder/a.txt',
+                file_info=mock.ANY,
+                progress_listener=mock.ANY,
+                encryption=None,
+                large_file_sha1=local_sha1 if should_be_incremental else None,
+            )
+        ]
+        # In Python 3.7 unittest.mock.call doesn't have `args` properly defined. Instead we have to take 1st index.
+        # TODO: use .args[0] instead of [1] when we drop Python 3.7
+        num_calls = len(bucket.mock_calls[0][1])
+        assert num_calls == 2 if should_be_incremental else 1, bucket.mock_calls[0]
+        if should_be_incremental:
+            # Order of indices: call index, pick arguments, pick first argument, first element of the first argument.
+            assert isinstance(bucket.mock_calls[0][1][0][0], CopySource)
+
 
 class TstEncryptionSettingsProvider(AbstractSyncEncryptionSettingsProvider):
     def __init__(self, source_encryption_setting, destination_encryption_setting):
