@@ -7,10 +7,9 @@
 # License https://www.backblaze.com/using_b2_code.html
 #
 ######################################################################
+from __future__ import annotations
 
 import pytest
-
-from b2sdk.exception import ResourceNotFound
 from apiver_deps_exception import (
     AlreadyFailed,
     B2Error,
@@ -22,7 +21,6 @@ from apiver_deps_exception import (
     DuplicateBucketName,
     FileAlreadyHidden,
     FileNotPresent,
-    interpret_b2_error,
     InvalidAuthToken,
     MissingPart,
     PartSha1Mismatch,
@@ -32,8 +30,11 @@ from apiver_deps_exception import (
     TransactionCapExceeded,
     Unauthorized,
     UnknownError,
+    interpret_b2_error,
 )
 
+from b2sdk._internal.exception import ResourceNotFound
+
 
 class TestB2Error:
     def test_plain_ascii(self):
